#ifndef INCL_TABLET_H
#define INCL_TABLET_H

#include "device.h"
#include "misc.h"
#include "if/ifcommon.h"

#include "pc8875.h"
#include "creator.h"
#include "mypad.h"
#include "mt20.h"
#include "cat.h"
#include "neos.h"
#include "TabletStatus.h"

#define	MODE_DEFAULT		0		// Disabled
#define	USEKEY1_DEFAULT		0		// Disabled
#define KEYPORT1_DEFAULT	9		// STOP KEY(Port 9/Bit 0)
#define	KEYBIT1_DEFAULT		0		
#define	USEKEY2_DEFAULT		0		// Disabled
#define KEYPORT2_DEFAULT	9		// ESC KEY(Port 9/Bit 7)
#define	KEYBIT2_DEFAULT		7		
#define	MOUSE_SENS_DEFAULT	100
#define	MOUSE_SENS_MAX		300
#define	MOUSE_SENS_MIN		10

#define	DATABUF_SIZE		4096

typedef struct {
	int mode;
	int usekey1;
	int keyport1;
	int keybit1;
	int usekey2;
	int keyport2;
	int keybit2;
	int mouse_sens;
} TABLET_CONFIG;

// ---------------------------------------------------------------------------

class Tablet : public Device
{
public:
	enum
	{
		reset= 0, out20, out21, out30, out40, out44, out80,
		in0x = 0, in20, in21, in45, in80, in81, inBx
	};
	enum TabletMode { disabled, ascii_binary, bcd, logitec, wacom, epson, mouse, hal, busmouse };

public:
	Tablet();
	~Tablet();
	
	bool Init(IIOAccess* access, IScheduler* _schduler, ITime* _tim, 
			  HINSTANCE hinst, uint prxrdy, uint prequest);
	void Release();

	bool Open(TABLET_CONFIG* pconfig);
	void Close();
	char* GetMessage();

	// IDevice Method
	const Descriptor* IFCALL GetDesc() const { return &descriptor; }
	
	// I/O port functions
	void IOCALL Reset(uint=0, uint=0);
	void IOCALL Out20(uint, uint);
	void IOCALL Out21(uint, uint);
	void IOCALL Out30(uint, uint);
	void IOCALL Out40(uint, uint);
	void IOCALL Out44(uint, uint);
	void IOCALL Out80(uint, uint);
	uint IOCALL In0x(uint);
	uint IOCALL In20(uint);
	uint IOCALL In21(uint);
	uint IOCALL In45(uint);
	uint IOCALL In80(uint);
	uint IOCALL In81(uint);
	uint IOCALL InBx(uint);

private:
	bool GetPacket(PACKET& packet);
	void ApplyConfig();
	void SetData(byte* data, uint count);
	uint ThreadMain();
	static uint __stdcall ThreadEntry(LPVOID arg);
	void IOCALL TimeEvent(uint);
	bool IsTabletEnabled() 
	{ 
		return (wintab_enabled && hctx != NULL);
	}
	bool IsRs232cEnabled()
	{ 
		return (wintab_enabled && (port30 & 0x20) != 0 && hctx != NULL);
	}

	enum Mode { clear=0, async, sync1, sync2, sync };

	Mode mode;

	IIOAccess* access;
	IScheduler* scheduler;
	ITime* tim;
	uint prxrdy;
	uint prequest;

	char message[256];
	char* tabletname;

	TABLET_CONFIG config;
	TabletStatus status;
	uint port30;

	CRITICAL_SECTION cs;
	bool		wintab_enabled;
	HCTX		hctx;
	LOGCONTEXT	lc;

	Pc8875		pc8875;
	Creator		creator;
	Mypad		mypad;
	Mt20		mt20;
	Cat			cat;
	Neos		neos;

	HWND		hwnd;
	HANDLE		hthread;
	uint		idthread;
	bool		running;
	bool		checking;
	bool		setting;
	byte		databuf[DATABUF_SIZE];
	int			statuscount;
	int			datacount;
	int			datapos;
	DWORD		tick_setdata;

	static const Descriptor descriptor;
	static const OutFuncPtr outdef[];
	static const InFuncPtr  indef[];
};

#endif // INCL_TABLET_H
