#include "headers.h"
#include "creator.h"

Creator::Creator()
{
	InitializeCriticalSection(&cs);
	Reset();
}

Creator::~Creator()
{
	DeleteCriticalSection(&cs);
}

void Creator::Reset()
{
	x		= 0;
	y		= 0;
	valid	= false;
	buttons	= 0;
	port80	= 0;
	port81	= 0;
}

void Creator::Open(LOGCONTEXT _lc)
{
	Reset();
	lc = _lc;
}

void Creator::Out80(uint data)
{
	EnterCriticalSection(&cs);

	switch(data)
	{
	case 0:
		port80  = ((x >> 8) & 0x0f);
		port80 |= ((buttons << 7) & 0x80);
		port81  = x & 0xff;
		break;
	case 1:
		port80  = ((y >> 8) & 0x0f);
		port80 |= ((buttons << 7) & 0x80);
		port81 = y & 0xff;
		break;
	}

	LeaveCriticalSection(&cs);
}

uint Creator::In80()
{
	return (valid ? 0x10 : 0x40) | port80;
}

uint Creator::In81()
{
	return port81;
}

void Creator::SetValue(PACKET& packet, bool _valid)
{
	EnterCriticalSection(&cs);

	const int max_x = 3099;
	const int max_y = 2099;
	valid = _valid;
	if (_valid)
	{
		x = Limit((max_x * packet.pkX) / lc.lcInExtX, max_x, 0);
		y = max_y - Limit((max_y * packet.pkY) / lc.lcInExtY, max_y, 0);
		buttons = packet.pkButtons;
	}

	LeaveCriticalSection(&cs);
}
