#include "headers.h"
#include "if/ifcommon.h"
#include "if/ifguid.h"
#include "Tablet.h"
#include "config.h"

#define EXTDEVAPI	__declspec(dllexport)

HINSTANCE hinst;

// ---------------------------------------------------------------------------

enum SpecialPort
{
	pint0 = 0x100, 
	pint1, pint2, pint3, pint4, pint5, pint6, pint7,
	pres,			// reset
	pirq,			// IRQ
	piack,			// interrupt acknowledgement
	vrtc,			// vertical retrace
	popnio,			// OPN ̓o̓|[g 1
	popnio2,		// OPN ̓o̓|[g 2 (A)
	psioin,			// SIO ֌W
	psioreq,
	ptimesync,
	portend
};

// ---------------------------------------------------------------------------

class TabletModule : public IModule
{
public:
	TabletModule();
	~TabletModule() {}

	bool Init(ISystem* system);
	void IFCALL Release();
	void* IFCALL QueryIF(REFIID) { return 0; }

private:
	Tablet tablet;
	ConfigTablet cfg;
	
	ISystem* sys;
	IIOBus* bus;
	IConfigPropBase* pb;
};

TabletModule::TabletModule()
: bus(0), pb(0)
{
	cfg.Init(hinst, &tablet);
}

bool TabletModule::Init(ISystem* _sys)
{
	sys = _sys;
	
	bus = (IIOBus*) sys->QueryIF(M88IID_IOBus1);
	pb = (IConfigPropBase*) sys->QueryIF(M88IID_ConfigPropBase);
	IIOAccess* access = (IIOAccess*) sys->QueryIF(M88IID_IOAccess1);
	IScheduler* sch = (IScheduler*) sys->QueryIF(M88IID_Scheduler);
	ITime* tim = (ITime*) sys->QueryIF(M88IID_Time);
	
	if (!bus || !pb || !access)
		return false;
	
	const static IIOBus::Connector c_tablet[] =
	{
		{ pres, IIOBus::portout, Tablet::reset },
		{ 0x20, IIOBus::portout, Tablet::out20 },
		{ 0x21, IIOBus::portout, Tablet::out21 },
		{ 0x30, IIOBus::portout, Tablet::out30 },
		{ 0x40, IIOBus::portout, Tablet::out40 },
		{ 0x44, IIOBus::portout, Tablet::out44 },
		{ 0x80, IIOBus::portout, Tablet::out80 },
		{ 0x00, IIOBus::portin,  Tablet::in0x  },
		{ 0x01, IIOBus::portin,  Tablet::in0x  },
		{ 0x02, IIOBus::portin,  Tablet::in0x  },
		{ 0x03, IIOBus::portin,  Tablet::in0x  },
		{ 0x04, IIOBus::portin,  Tablet::in0x  },
		{ 0x05, IIOBus::portin,  Tablet::in0x  },
		{ 0x06, IIOBus::portin,  Tablet::in0x  },
		{ 0x07, IIOBus::portin,  Tablet::in0x  },
		{ 0x08, IIOBus::portin,  Tablet::in0x  },
		{ 0x09, IIOBus::portin,  Tablet::in0x  },
		{ 0x0a, IIOBus::portin,  Tablet::in0x  },
		{ 0x0b, IIOBus::portin,  Tablet::in0x  },
		{ 0x0c, IIOBus::portin,  Tablet::in0x  },
		{ 0x0d, IIOBus::portin,  Tablet::in0x  },
		{ 0x0e, IIOBus::portin,  Tablet::in0x  },
		{ 0x20, IIOBus::portin,  Tablet::in20  },
		{ 0x21, IIOBus::portin,  Tablet::in21  },
		{ 0x45, IIOBus::portin,  Tablet::in45  },
		{ 0x80, IIOBus::portin,  Tablet::in80  },
		{ 0x81, IIOBus::portin,  Tablet::in81  },
		{ 0xb0, IIOBus::portin,  Tablet::inBx  },
		{ 0xb1, IIOBus::portin,  Tablet::inBx  },
		{ 0xb2, IIOBus::portin,  Tablet::inBx  },
		{ 0xb3, IIOBus::portin,  Tablet::inBx  },
		{ 0, 0, 0 }
	};
	if (!tablet.Init(access, sch, tim, hinst, pint0, psioin) || !bus->Connect(&tablet, c_tablet))
		return false;

	pb->Add(&cfg);

	tablet.Open(cfg.GetConfig());

	return true;
}

void TabletModule::Release()
{
	tablet.Release();

	if (bus)
		bus->Disconnect(&tablet);
	if (pb)
		pb->Remove(&cfg);

	delete this;
}

// ---------------------------------------------------------------------------

//	Module 쐬
extern "C" EXTDEVAPI IModule* __cdecl M88CreateModule(ISystem* system)
{
	TabletModule* module = new TabletModule;
	
	if (module)
	{
		if (module->Init(system))
			return module;
		delete module;
	}
	return 0;
}

BOOL APIENTRY DllMain(HANDLE hmod, DWORD rfc, LPVOID)
{
	switch (rfc)
	{
	case DLL_PROCESS_ATTACH:
		hinst = (HINSTANCE) hmod;
		break;
	
	case DLL_THREAD_ATTACH:
		break;
	
	case DLL_THREAD_DETACH:
		break;
	
	case DLL_PROCESS_DETACH:
		break;
	}
	return true;
}
