#include "headers.h"
#include "neos.h"

Neos::Neos()
{
	InitializeCriticalSection(&cs);
	Reset();
}

Neos::~Neos()
{
	DeleteCriticalSection(&cs);
}

void Neos::Reset()
{
	strobe	= false;
	mode	= 0;
	x		= 0;
	y		= 0;
	valid	= false;
	buttons	= 0;
	reg		= 0;
	port45	= 0;
}

void Neos::Open(int _sensitivity, LOGCONTEXT _lc)
{
	Reset();
	lc = _lc;
	sensitivity = _sensitivity;
}

void Neos::Out40(uint data, int tim)
{
	bool current = ((data & 0x40) != 0);

	if (!strobe && current)
	{
		switch (mode)
		{
		case 0:
			EnterCriticalSection(&cs);
			vx = valid && sv ? Limit((sx - x) * sensitivity / 100, 127, -128) : 0;
			vy = valid && sv ? Limit((y - sy) * sensitivity / 100, 127, -128) : 0;
			sx = x;
			sy = y;
			sv = valid;
			port45 = ((vx >> 4) & 0x0f);
			mode = 1;
			LeaveCriticalSection(&cs);
			break;
		case 2:
			port45 = ((vy >> 4) & 0x0f);
			mode = tim - tick > 18 ? 0 : 3;
			break;
		}
		tick = tim;
	}
	else if (strobe && !current)
	{
		switch(mode)
		{
		case 1:
			port45 = (vx & 0x0f);
			mode = tim - tick > 18 ? 0 : 2;
			break;
		case 3:
			port45 = (vy & 0x0f);
			mode = 0;
		}
		tick = tim;
	}
	strobe = current;
}

void Neos::Out44(uint data)
{
	reg = data;
}

uint Neos::In45()
{
	uint ret = 0xff;
	switch(reg)
	{
	case 0x0e:
		ret = port45;
		break;
	case 0x0f:
		if (valid)
		{
			ret = 0xfc | (~buttons & 0x03);
		}
		break;
	}
	return ret;
}

void Neos::SetValue(PACKET& packet, bool _valid)
{
	EnterCriticalSection(&cs);

	valid	= _valid;
	x		= packet.pkX;
	y		= packet.pkY;
	buttons	= packet.pkButtons;

	LeaveCriticalSection(&cs);
}
